/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package eu.stork.support;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.util.HashMap;

//import javax.annotation.processing.FilerException;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;


/**
 * @author INDRA
 *
 */
public interface IPDFComponent {
		
//        /** Writes a field into a specified pdfStamper.
//         * @param fieldToWrite Name of field to write. 
//         * @param fieldValue Value of field to write. 
//         * @param pdfStamper_ IText PDFStamper.
//         * @throws IOException
//         * @throws DocumentException
//         */
        /** Writes field into a PDFStamper. This function let write a single field into a PDFStamper.  
         * @param fieldToWrite Name of field to be written.  
         * @param fieldValue Value of field to be written.
         * @param pdfStamper_ IText PDFStamper object. 
         * @throws IOException
         * @throws DocumentException
         */
        public void writeField(String fieldToWrite, String fieldValue, PdfStamper pdfStamper_)throws IOException, DocumentException;
        
//        /**
//         * Writes the fields 
//         * @param pdfStamper_ IText PDFStamper object. 
//         * @throws IOException
//         * @throws DocumentException
//         */
      //  public void writeFields(PdfStamper pdfStamper) throws IOException, DocumentException;

        /**
         * Obtains the content of a field from a PDF document
         * @param fieldToRead Name of the field to obtain. 
         * @param pdfStamper_ IText PDFStamper object. 
         * @return The value of the field.
         * @throws SignatureException
         * @throws CertificateException
         */
        public String readField(String fieldToRead, PdfStamper pdfStamper_) throws SignatureException, CertificateException;        

//        /**
//         * Signs a PDF document. 
//         * @param certPath_ Path to keystore.
//         * @param passwordKeyStore Keystore password. 
//         * @param keyAlias Key alias. 
//         * @param pdfFilled_ Path to document to be signed.
//         * @param pdfToSign_ Path to document signed. 
//         * @return if the document has been signed correctly. 
//         */
      //  public boolean signPDFDocument(String certPath_, String passwordKeyStore, String keyAlias, String pdfFilled_, String pdfToSign_);

        /**
         * Checks the signature of PDF document. 
         * @param pr_ PDFReader object which represents the IText document. 
         * @return If the signature of document is valid.  
         * @throws SignatureException
         * @throws CertificateException
         */
        public boolean checkPDFDocumentSignature(PdfReader pr_) throws SignatureException, CertificateException;
        
        /**
         * Saves the PDF document. 
         * @param pdfReader PDFReader object which represents the IText document. 
         * @param outputFile Output file. 
         * @throws DocumentException
         * @throws IOException
         */
        public void saveDocument(PdfReader pdfReader, String outputFile) throws DocumentException,IOException;
        
        
        /**
     	 * Create the PDFDocument. Don't save the file	
         * @param pdfReader
         * @param outputFile
         * @return ByteArrayOutputStream
         * @throws DocumentException
         * @throws IOException
         */
        
        public ByteArrayOutputStream getOutputStream(PdfReader pdfReader, String outputFile)throws DocumentException, IOException;
}
