/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

 package eu.stork.support;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfSignatureAppearance;
import com.itextpdf.text.pdf.PdfStamper;


/**
 * This class is used to make the signature of the PDF document. 
 *
 */
public class PDFSigner {
	 private String inputPDF = "";
     private String outputPDF = "";
     private StorkCertificate cert;
     private Metadata metadata;
     
     /* Modificacion mia aadido outputStream*/
     
     private ByteArrayOutputStream outPdf;
     
     /* Modificacion mia aadido outputStream*/
     
     /**
      * @param input ByteArrayOutputStream to input file to be signed
      * @param cert
      * @param md 
      */
     
     public PDFSigner(ByteArrayOutputStream input, StorkCertificate cert ,Metadata md)
     {
         this.outPdf = input;
         this.cert = cert;
         this.metadata = md;
     }
     

     /**
     * @param input Path to input file to be signed
     * @param output Path to input file signed
     */
    public PDFSigner(String input, String output)
     {
         this.inputPDF = input;
         this.outputPDF = output;
     }
     /**
     * @param input Path to input file to be signed
     * @param output Path to input file signed
     * @param cert
     */
    public PDFSigner(String input, String output, StorkCertificate cert)
     {
         this.inputPDF = input;
         this.outputPDF = output;
         this.cert = cert;
     }
     /**
     * @param input Path to input file to be signed
     * @param output Path to input file signed
     * @param md
     */
    public PDFSigner(String input, String output, Metadata md)
     {
         this.inputPDF = input;
         this.outputPDF = output;
         this.metadata = md;
     }

     /**
     * @param input Path to input file to be signed
     * @param output Path to input file signed
     * @param cert
     * @param md
     */
    public PDFSigner(String input, String output, StorkCertificate cert, Metadata md)
     {
         this.inputPDF = input;
         this.outputPDF = output;
         this.cert = cert;
         this.metadata = md;
     }

 
     /// <summary>
     /// Method to sign a PDF document
     /// </summary>
     /// <param name="SigReason">Signatures reason</param>
     /// <param name="SigContact">Signers Contact</param>
     /// <param name="SigLocation">Signatures Local</param>
     /// <param name="visible">Signatures visibility</param>
     /**
     * @param SigReason Signature reaseon
     * @param SigContact Signature contact
     * @param SigLocation Signature location
     * @param visible Signature visibility. 
     */
    public void sign(String SigReason, String SigContact, String SigLocation, boolean visible)
     {
         PdfReader reader= null;
         PdfStamper st=null;
		try {
			reader = new PdfReader(this.inputPDF);

	       //Activate MultiSignatures
         st = PdfStamper.createSignature(reader, new FileOutputStream(this.outputPDF), '\0', null, true);            
		
         st.setMoreInfo(this.metadata.getMetaData());
         st.setXmpMetadata(this.metadata.getStreamedMetaData());
         PdfSignatureAppearance sap = st.getSignatureAppearance();

         sap.setCrypto(this.cert.getKey(), this.cert.getCertificate(), null, PdfSignatureAppearance.WINCER_SIGNED);
         sap.setReason(SigReason);
         sap.setContact(SigContact);
         sap.setLocation(SigLocation);
         sap.setCertificationLevel(PdfSignatureAppearance.CERTIFIED_NO_CHANGES_ALLOWED);
         if (visible)
             sap.setVisibleSignature(new Rectangle(235, 320, 355, 270), 1, null);                

         reader.close();
         st.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			if (reader!=null) 
				reader.close();
			
		
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			if (reader!=null) 
				reader.close();
		
			e.printStackTrace();
		}
  
     }
    
    
    
    /**
     * @param SigReason
     * @param SigContact
     * @param SigLocation
     * @param visible
     * @return
     */
    public OutputStream signOutputStream(String SigReason, String SigContact, String SigLocation, boolean visible){
    	
    	 PdfReader reader= null;
         PdfStamper st=null;
         ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {

		 reader = new PdfReader(new ByteArrayInputStream(this.outPdf.toByteArray()));

         st = PdfStamper.createSignature(reader, out, '\0', null, true);            
		
         st.setMoreInfo(this.metadata.getMetaData());
         st.setXmpMetadata(this.metadata.getStreamedMetaData());
         PdfSignatureAppearance sap = st.getSignatureAppearance();

         sap.setCrypto(this.cert.getKey(), this.cert.getCertificate(), null, PdfSignatureAppearance.WINCER_SIGNED);
         sap.setReason(SigReason);
         sap.setContact(SigContact);
         sap.setLocation(SigLocation);
         sap.setCertificationLevel(PdfSignatureAppearance.CERTIFIED_NO_CHANGES_ALLOWED);
         if (visible)
             sap.setVisibleSignature(new Rectangle(235, 320, 355, 270), 1, null);                

         reader.close();
         st.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			if (reader!=null) 
				reader.close();
			
		
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			if (reader!=null) 
				reader.close();
		
			e.printStackTrace();
		}
    	
    	return out;
    }  

}
