/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package eu.stork.support;
import java.io.IOException;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import eu.stork.support.pkcs11.PKCS11Support;


/**
 * This class represents the data used to specify the Certificate which will be used to sign the PDF document. 
 *
 */
public class StorkCertificate {


      private String path = "";
      private String password = "";
      private String alias="";
      private PKCS11Support pkcs11support;
      public PKCS11Support getPkcs11support() {
		return pkcs11support;
	}
	public void setPkcs11support(PKCS11Support pkcs11support) {
		this.pkcs11support = pkcs11support;
	}
	public String getAlias() {
		return alias;
	}
	public void setAlias(String alias) {
		this.alias = alias;
	}
	private PrivateKey key;
      private X509Certificate[] chain;
      public PrivateKey getKey() {
		return key;
	}
	public void setKey(PrivateKey key) {
		this.key = key;
	}
	public X509Certificate[] getChain() {
		return chain;
	}
	public void setChain(X509Certificate[] chain) {
		this.chain = chain;
	}
	public String getPath() {
		return path;
	}
	public void setPath(String path) {
		this.path = path;
	}
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public Certificate[] getCertificate() {
		return certificate;
	}
	public void setCertificate(Certificate[] certificate) {
		this.certificate = certificate;
	}
	private Certificate[] certificate;

      private void processCert() throws Exception
      {
     
          
          java.security.KeyStore kstore = KeyStore.getInstance("jks");
        
        // get user password and file input stream
        char[] pwd = password.toCharArray();
    
        java.io.FileInputStream fis = null;
        try {
            fis = new java.io.FileInputStream(path);
            kstore.load(fis, pwd);
            if (kstore.containsAlias(alias)) {
           	 
          	  this.setKey((PrivateKey)kstore.getKey(alias, pwd));
            }
            Certificate[] certChain = kstore.getCertificateChain(alias);
             
            if (certChain!=null) {
            	X509Certificate[] certX509Chain = new X509Certificate[certChain.length];
            
	            for (int i=0;i<certChain.length;i++) {
	            	certX509Chain[i]= (X509Certificate)certChain[i];
	            }
	            this.setChain(certX509Chain);
	            this.setCertificate(certX509Chain);
            }
            
            
        } catch (IOException ex) {
        	ex.printStackTrace();
        }
        finally {
            if (fis != null) {
                fis.close();
            }
        }
        


         
      
          
      }
      /**
       * Instantiates a StorkCertificate.
       * @param cpath Path to keystore.
       * @throws Exception
       */
      public StorkCertificate(String cpath) throws Exception
      {
          this.path = cpath;
          this.processCert();
      }
      /**
       * Instantiates a StorkCertificate.
     * @param cpath Path to keystore.
     * @param cpassword Keystore password.
     * @throws Exception
     */
    public StorkCertificate(String cpath, String cpassword) throws Exception
      {
          this.path = cpath;
          this.password = cpassword;
          this.processCert();
      }
    /**
     * Instantiates a StorkCertificate.
     * @param cpath Path to keystore.
     * @param cpassword Keystore password.
     * @param alias Key alias.
     * @throws Exception
     */
      public StorkCertificate(String cpath, String cpassword,String alias) throws Exception
      {
          this.path = cpath;
          this.password = cpassword;
          this.alias=alias;
          this.processCert();
      }
      /**
       * Instantiates a StorkCertificate object using a PKCS11Support class. 
     * @param support PKCS11Support class which encapsulates configuration data for PKCS11 standard.
     */
    public StorkCertificate(PKCS11Support support) {
    	  this.pkcs11support=support;
    	  this.processCert(support);
    	  
      }
	private void processCert(PKCS11Support support) {
		
		 
	        
	           	           	 
	          	  this.setKey(support.getPrivateKey());
	          	  this.setChain(support.getCertificateChain());
	          	  this.setCertificate(support.getCertificateChain());
		
	}
}
