/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package eu.stork.support;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.AcroFields;
import com.itextpdf.text.pdf.PdfPKCS7;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;


/**
 * @author jaalbares
 *
 */
public class StorkPDF implements IPDFComponent  {
	 
	public void saveDocument(PdfReader pdfReader, String outputFile)
			throws DocumentException, IOException{
		 //Populate PDF Template
      
		 
        PdfStamper pdfStamper = new PdfStamper(pdfReader, new FileOutputStream(outputFile));

         writeFields(pdfStamper);

         // flatten the form to remove editting options, set it to false
         // to leave the form open to subsequent manual edits
         pdfStamper.setFormFlattening(false);
         
         pdfStamper.close();       
		
	}
	 
	 
	public ByteArrayOutputStream getOutputStream(PdfReader pdfReader, String outputFile)throws DocumentException, IOException{
			
			ByteArrayOutputStream outPdf = new ByteArrayOutputStream();
			
			PdfStamper pdfStamper = new PdfStamper(pdfReader, outPdf);
			
			writeFields(pdfStamper);
	        
			pdfStamper.setFormFlattening(false);
			
			pdfStamper.close();
			
			return outPdf;
	}

	private PDFSigner pdfSigner;
	 
	 private HashMap<String, String> fields = new HashMap<String, String>();

	 private String Stork_surname; 
     private String stork_eIdentifier;
     private String Stork_givenName;
     private String Stork_dateOfBirth; 
     private String stork_textResidence;
     private String stork_authorTest;



     public String getStork_authorTest() {
		return stork_authorTest;
	}

	public void setStork_authorTest(String storkAuthorTest) {
		stork_authorTest = storkAuthorTest;
		fields.put("stork_authorTest", storkAuthorTest);
	}

	public PDFSigner getPdfSigner() {
		return pdfSigner;
	}

	public void setPdfSigner(PDFSigner pdfSigner) {
		this.pdfSigner = pdfSigner;
	}

	/**
	 * Gets E-Identifier
	 * @return E-Identifier
	 */
	public String getStork_eIdentifier() {
		return this.stork_eIdentifier;
	}

	/**
	 * Sets E-Identifier
	 * @param storkEIdentifier E-Identifier. 
	 */
	public void setStork_eIdentifier(String storkEIdentifier) {
		stork_eIdentifier = storkEIdentifier;
//		 "stork_eIdentifier", storkPDF.getStork_eIdentifier(), pdfStamper);
//       pdfInterface.writeField("stork_surname", storkPDF.getStork_surname(), pdfStamper);
//       pdfInterface.writeField("stork_givenName", storkPDF.getStork_givenName(), pdfStamper);
//       pdfInterface.writeField("stork_dateOfBirth", storkPDF.getStork_dateOfBirth(), pdfStamper);
//       pdfInterface.writeField("stork_textResidence", storkPDF.getStork_textResidence(), pdfStamper);
//       pdfInterface.writeField("stork_countryCode",
		fields.put("stork_eIdentifier", storkEIdentifier);
	}

	/**
	 * Gets the surname
	 * @return surname
	 */
	public String getStork_surname() {
		return Stork_surname;
	}
	
	/**
	 * Sets the surname
	 * @param storkSurname surname.
	 */
	public void setStork_surname(String storkSurname) {
		Stork_surname = storkSurname;
		fields.put("stork_surname", storkSurname);
	}



	/**
	 * Returns Stork given name
	 * @return Stork given name
	 */
	public String getStork_givenName() {
		return Stork_givenName;
	}

	/**
	 * Sets the Stork Given Name
	 * @param storkGivenName Stork Given Name
	 */
	public void setStork_givenName(String storkGivenName) {
		Stork_givenName = storkGivenName;
		fields.put("stork_givenName",storkGivenName);
	}

	/**
	 * Gets the date of birth. 
	 * @return the date of birth 
	 */
	public String getStork_dateOfBirth() {
		return Stork_dateOfBirth;
	}

	/**
	 * Sets the date of birth. 
	 * @param storkDateOfBirth date of birth. 
	 */
	public void setStork_dateOfBirth(String storkDateOfBirth) {
		Stork_dateOfBirth = storkDateOfBirth;
		fields.put("stork_dateOfBirth", storkDateOfBirth);
	}

	/**
	 * Gets the text residence. 
	 * @return Text residence. 
	 */
	public String getStork_textResidence() {
		return stork_textResidence;
	}

	/**
	 * Sets text residence.
	 * @param storkTextResidence text Residence
	 */
	public void setStork_textResidence(String storkTextResidence) {
		stork_textResidence = storkTextResidence;
		fields.put("stork_textResidence", storkTextResidence);
	}

	/** Gets the country code. 
	 * @return the country code. 
	 */
	public String getStork_countryCode() {
		return stork_countryCode;
	}

	/**
	 * Sets the country code. 
	 * @param storkCountryCode country code. 
	 */
	public void setStork_countryCode(String storkCountryCode) {
		stork_countryCode = storkCountryCode;
		fields.put("stork_countryCode", storkCountryCode);
		
	}

	public String getStork_signature() {
		return stork_signature;
	}

	public void setStork_signature(String storkSignature) {
		stork_signature = storkSignature;
		
	}


	/**
	 * Stork Country Code
	 */
	private String stork_countryCode;

     /**
     * Stork Signature.
     */
    private String stork_signature;



    
     /// <summary>
     /// Constructor to StorkPDF object
     /// </summary>
     /// <param name="eIdentifier_">Personal Stork eIdentifier</param>
     /// <param name="givenName_">First User Name</param>
     /// <param name="surname_">User Surname</param>
     /// <param name="textResidence_">User Address</param>
     /// <param name="dateOfBirth_">User Birthdays</param>
     /// <param name="countryCode_">User country codes</param>
     /**
     * @param eIdentifier_ E-Identifier
     * @param givenName_ Given name
     * @param surname_ Surname.
     * @param textResidence_ Text residence
     * @param dateOfBirth_ Date of Birth
     * @param countryCode_ Country code
     */
    public StorkPDF(String eIdentifier_, String givenName_, String surname_, String textResidence_, String dateOfBirth_, String countryCode_,String authorTest_)
     {
         this.setStork_eIdentifier(eIdentifier_);
//         "stork_eIdentifier", storkPDF.getStork_eIdentifier(), pdfStamper);
//         pdfInterface.writeField("stork_surname", storkPDF.getStork_surname(), pdfStamper);
//         pdfInterface.writeField("stork_givenName", storkPDF.getStork_givenName(), pdfStamper);
//         pdfInterface.writeField("stork_dateOfBirth", storkPDF.getStork_dateOfBirth(), pdfStamper);
//         pdfInterface.writeField("stork_textResidence", storkPDF.getStork_textResidence(), pdfStamper);
//         pdfInterface.writeField("stork_countryCode",
         this.setStork_givenName(givenName_);
         this.setStork_surname(surname_);
         this.setStork_textResidence(textResidence_);
         this.setStork_dateOfBirth(dateOfBirth_);
         this.setStork_countryCode(countryCode_);
         this.setStork_authorTest(authorTest_);
     }

     public StorkPDF() {
		super();
		// TODO Auto-generated constructor stub
	}

	/// <summary>
     /// Method to write a PDF field
     /// </summary>
     /// <param name="fieldToWrite">Field name</param>
     /// <param name="fieldValue">Field value</param>
     /// <param name="pdfTemplate_">PDF template document </param>
     /// <param name="newFile_">PDF filled with all fields</param>
     /// <param name="pdfStamper_">PDF Stamper object</param>
     public void writeField(String fieldToWrite, String fieldValue, PdfStamper pdfStamper_) throws IOException, DocumentException
     {          
         AcroFields pdfFormFields = pdfStamper_.getAcroFields();
         pdfFormFields.setField(fieldToWrite, fieldValue);
     }

     /// <summary>
     /// Method to read from a PDF field
     /// </summary>
     /// <returns>field name to read</returns>
     public String readField(String fieldToRead, PdfStamper pdfStamper_)
     {
         AcroFields pdfFormFields = pdfStamper_.getAcroFields();
         return pdfFormFields.getField(fieldToRead);
     }


     /// <summary>
     /// Method to sig document
     /// </summary>
     /// <param name="certPath_">Path to cerificate file</param>
     /// <param name="pdfFilled_">PDF document with all fields filled</param>
     /// <param name="pdfToSign_">PDF to sign</param>
     /// <returns></returns>
     private boolean signPDFDocument(String certPath_, String passwordKeyStore, String keyAlias, String pdfFilled_, String pdfToSign_)
     {
         StorkCertificate myCert = null;
         try
         {
             //Hard Coded.
             myCert = new StorkCertificate(certPath_, passwordKeyStore,keyAlias);                
         }
         catch (Exception ex)
         {
             //TODO
             return false;
         }

         //Adding Meta Data
         Metadata metadata = new Metadata();
         metadata.setAuthr("Test Service Provider");
         metadata.setProducer("Service Provider");
         metadata.setSubject("Change of address");
         metadata.setTitle("Change of address");

         try
         {
             pdfSigner = new PDFSigner(pdfFilled_, pdfToSign_, myCert,metadata);

             pdfSigner.sign("I am the author", "PhoneNumber", "Office", true);
         }
         catch(Exception ex)
         {
             //TODO
             return false;
         }

         return true;
     }


     

    public boolean checkPDFDocumentSignature(PdfReader pr_) throws SignatureException, CertificateException 
     {     
             PdfPKCS7 pk= null;

             if (pr_ != null)
             {
                 AcroFields af = pr_.getAcroFields();
                 if (af != null)
                 {
                     ArrayList names = af.getSignatureNames();
                     for (int k = 0; k < names.size(); ++k)
                     {
                         String name = (String)names.get(k);

                         //Verify if signature covers whole document
                         if (af.signatureCoversWholeDocument(name))
                         {
                             // Verify document signature and if document was changed
                             pk = af.verifySignature(name);
                             Calendar cal = pk.getSignDate();

                             ((X509Certificate)pk.getCertificates()[0]).checkValidity(cal.getTime());
                                 if (!pk.verify())
                                     return false;

                             //
                             Object fails = PdfPKCS7.verifyCertificate((X509Certificate) pk.getCertificates()[0], null, cal);

                             if (fails != null)
                                 return false;
                             // indra
                             else
                            	 return true;
//                             byte[] myCertificateBytes = pk.getCertificates()[0].getEncoded();
//                             
//                             X509Certificate cert = (X509Certificate) CertificateFactory.getInstance("X509").generateCertificate(new ByteArrayInputStream(myCertificateBytes));
                             
                          
                             
//                             try {
//								java.security.KeyStore trustedCertsStore = KeyStore.getInstance("jks");
////                           
////                             KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
////                           
////                               // get user password and file input stream
//								   char[] password = "mytrustedcert".toCharArray();
////                           
//								   java.io.FileInputStream fis = null;
//								   try {
//								       fis = new java.io.FileInputStream("C:/PROYECTOS/STORK/pdfGenerator/ExternalRef/mytrustedcerts.jks");
//								       trustedCertsStore.load(fis, password);
//								       X509Certificate cert2 = (X509Certificate)trustedCertsStore.getCertificate("mytrustedcert");
//								       cert2.getSerialNumber();
//								       cert.
//								       
//								   } finally {
//								       if (fis != null) {
//								           fis.close();
//								       }
//								   }
//							} catch (KeyStoreException e) {
//								// TODO Auto-generated catch block
//								e.printStackTrace();
//							} catch (FileNotFoundException e) {
//								// TODO Auto-generated catch block
//								e.printStackTrace();
//							} catch (NoSuchAlgorithmException e) {
//								// TODO Auto-generated catch block
//								e.printStackTrace();
//							} catch (IOException e) {
//								// TODO Auto-generated catch block
//								e.printStackTrace();
//							}
//                             
                             
                             //Verify if PDF certificate is in my MS Thrust List of Certificates
//                             X509Store store = new X509Store(storeName.My, StoreLocation.CurrentUser);
//                             store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);
//
//                             if (st.getCertificates.Contains(cert))
//                                 return true;
//                             else
//                                 return false;
                         }
                     }
                 }
             }

         return false;
     }
     
     /**
      * Writes the fields saved from StorkPDF Object into PDFStamper. 
      * @param pdfStamper IText PDFStamper Object
      * @throws IOException
      * @throws DocumentException
      */
    public void writeFields(PdfStamper pdfStamper) throws IOException, DocumentException{
    	    AcroFields pdfFormFields = pdfStamper.getAcroFields();
    	    
    	    	Iterator<String> it = fields.keySet().iterator();
    	    	while (it.hasNext()) {
    	    		String key= it.next();
    	    		pdfFormFields.setField(key, fields.get(key));		
    	    	}
    	    
            

    	 
     }

}
