/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package eu.stork.support;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;

public class StorkPDFOutputStream {

    private String pdfTemplate = "";
    private String newFile;
    private String certPath;

    private PdfReader pdfReader;
    private String pdfSigned;
    private IPDFComponent pdfInterface;

    private String keystore;
     


    public String getKeystore() {
		return keystore;
	}


	public void setKeystore(String keystore) {
		this.keystore = keystore;
	}


	private void Form1_Load() throws Exception
    {

        //Init
        pdfTemplate = "STORK_coA_Template.pdf";
   
        certPath= "C:\\KeyStorePdfStork\\scsp-keystore.jks";
        
      
        // Create a new PDF reader based on the PDF template document
        pdfReader = new PdfReader(pdfTemplate);

        //Populate PDF Template
        pdfInterface = new StorkPDF("0123456789", "Carlos", "Santana", "Rua da Prosa n14B", "1990 July 10th", "0123","Angel");
              
        ByteArrayOutputStream bytesPdf = pdfInterface.getOutputStream(pdfReader, newFile);
        
        //Sign PDF
 
        IPDFComponent sample = new StorkPDF();
        StorkCertificate sc = new StorkCertificate(certPath, "changeit","testeacn");
        
        
        PDFSigner pdfSigner = new PDFSigner(bytesPdf,sc,new Metadata());
        
        ByteArrayOutputStream bytesPdfSalida =(ByteArrayOutputStream)pdfSigner.signOutputStream("Test Reason", "Test Contact", "test location", true);

        //Validate PDF
        if (sample.checkPDFDocumentSignature(new PdfReader(bytesPdfSalida.toByteArray())))
        {
            System.out.println("**Certificate ok!" );
        }
        else
        {
            System.out.println("**Certificate failed!" );
        }
        
        ListFieldNames(bytesPdfSalida); 

        System.out.println("**Sucess - File dumped - " + pdfSigned);
    }


    private void ListFieldNames(OutputStream pdfOut) throws Exception
    {
        // Title the form
   
        //Read PDF to mem
        pdfReader = new PdfReader(((ByteArrayOutputStream)pdfOut).toByteArray());
        
        PdfStamper outStamper = new PdfStamper(pdfReader, new FileOutputStream("c://test_stork_pdf"));

        System.out.println("**Listing fieds" );
        //Print fields
        System.out.println("\tstork_eIdentifier - " + pdfInterface.readField("stork_eIdentifier", outStamper));
        System.out.println("\tstork_surname - " + pdfInterface.readField("stork_surname", outStamper) );
        System.out.println("\tstork_givenName - " + pdfInterface.readField("stork_givenName", outStamper));
        System.out.println("\tstork_dateOfBirth - " + pdfInterface.readField("stork_dateOfBirth", outStamper));
        System.out.println( "\tstork_textResidence - " + pdfInterface.readField("stork_textResidence", outStamper));
        System.out.println("\tstork_countryCode - " + pdfInterface.readField("stork_countryCode", outStamper));
        
        outStamper.getWriter().flush();
        outStamper.close();

    }
    
    
    /// <summary>
    /// Method to fill PDF 
    /// </summary>
    private void FillForm() throws  DocumentException, IOException
    {
//        storkPDF = new StorkPDF("0123456789", "Carlos", "Santana", "Rua da Prosa n14B", "1990 July 10th", "0123");
//        pdfInterface = storkPDF;
//        pdfStamper = new PdfStamper(pdfReader, new FileOutputStream(newFile));
//
////        pdfInterface.writeField("stork_eIdentifier", storkPDF.getStork_eIdentifier(), pdfStamper);
////        pdfInterface.writeField("stork_surname", storkPDF.getStork_surname(), pdfStamper);
////        pdfInterface.writeField("stork_givenName", storkPDF.getStork_givenName(), pdfStamper);
////        pdfInterface.writeField("stork_dateOfBirth", storkPDF.getStork_dateOfBirth(), pdfStamper);
////        pdfInterface.writeField("stork_textResidence", storkPDF.getStork_textResidence(), pdfStamper);
////        pdfInterface.writeField("stork_countryCode", storkPDF.getStork_countryCode(), pdfStamper);
//        pdfInterface.writeFields(pdfStamper);
//
//        // flatten the form to remove editting options, set it to false
//        // to leave the form open to subsequent manual edits
//        pdfStamper.setFormFlattening(false);
//
//        pdfStamper.close();
    }

	
	public static void main(String[] args) {
   	 try {
			new StorkPDFOutputStream().Form1_Load();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
	}
	
	
}
