// This file is part of the StorkPDF software library.
// Copyright (c) 2010,2011 European Commission
// Authors: Agncia para a Modernizao Administrativa (www.ama.pt), et al.
//
// This work  is Open Source and licensed by the European Commission under the
// conditions of the Affero General Public License v3 
// (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
// in compliance with the License. Unless required by applicable law or agreed 
// to in writing, software distributed under the License is distributed on an 
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.

 using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using iTextSharp;
using iTextSharp.text;
using iTextSharp.text.pdf;
using iTextSharp.text.xml;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.Security;
using System.Security.Cryptography;
using iTextSharpSign;

namespace PdfGenerator
{
    public partial class Form1 : Form
    {
        #region Properties

        private StorkPDF storkPDF;
        private PdfStamper pdfStamper;
        private string pdfTemplate = string.Empty;
        private string newFile;
        private string certPath;
        private PdfReader pdfReader;
        private string pdfSigned;
        private IPDFComponent pdfInterface;

        public string KeyStore { get; set; }

        #endregion

        #region Methods

        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {

            //Init
            pdfTemplate = @"ExternalRef\STORK_coA_Template.pdf";
            newFile = @"ExternalRef\STORK_coA.pdf";
            pdfSigned = @"ExternalRef\STORK_coA_Signed.pdf";
            certPath = @"ExternalRef\certificadoChavePublicaePrivada.pfx";
            X509Certificate2 certificate = new X509Certificate2(certPath, "teste");

            textBox1.Text += "**Init" + Environment.NewLine;

            // Create a new PDF reader based on the PDF template document
            pdfReader = new PdfReader(pdfTemplate);

            //Populate PDF Template
            FillForm();            
            
            //Sign PDF
            IPDFComponent sample = new StorkPDF();
            sample.SignPDFDocument(certPath, newFile, pdfSigned);


            //Validate PDF
            if (sample.CheckPDFDocumentSignature(new PdfReader(pdfSigned)))
            {
                textBox1.Text += "**Certificate ok!" + Environment.NewLine;
            }
            else
            {
                textBox1.Text += "**Certificate failed!" + Environment.NewLine;
            }

            ListFieldNames(pdfSigned);

            textBox1.Text += "**Sucess - File dumped - " + pdfSigned + Environment.NewLine;
        }


        /// <summary>
        /// List all of the form fields into a textbox.  The
        /// form fields identified can be used to map each of the
        /// fields in a PDF.
        /// </summary>
        private void ListFieldNames(string pdfSigned_)
        {
            // Title the form
            this.Text += " - " + pdfSigned_;
           
            //Read PDF to mem
            pdfReader = new PdfReader(pdfSigned_);
            MemoryStream m = new MemoryStream();
            PdfStamper outStamper = new PdfStamper(pdfReader, m);

            textBox1.Text += "**Listing fieds" + Environment.NewLine;
            //Print fields
            textBox1.Text += "\tstork_eIdentifier - " + storkPDF.ReadField("stork_eIdentifier", outStamper) + Environment.NewLine;
            textBox1.Text += "\tstork_surname - " + storkPDF.ReadField("stork_surname", outStamper) + Environment.NewLine;
            textBox1.Text += "\tstork_givenName - " + storkPDF.ReadField("stork_givenName", outStamper) + Environment.NewLine;
            textBox1.Text += "\tstork_dateOfBirth - " + storkPDF.ReadField("stork_dateOfBirth", outStamper) + Environment.NewLine;
            textBox1.Text += "\tstork_textResidence - " + storkPDF.ReadField("stork_textResidence", outStamper) + Environment.NewLine;
            textBox1.Text += "\tstork_countryCode - " + storkPDF.ReadField("stork_countryCode", outStamper) + Environment.NewLine;

        }


        /// <summary>
        /// Method to fill PDF 
        /// </summary>
        private void FillForm()
        {
            storkPDF = new StorkPDF("0123456789", "Carlos", "Santana", "Rua da Prosa n14B", "1990 July 10th", "0123");
            pdfInterface = storkPDF;
            pdfStamper = new PdfStamper(pdfReader, new FileStream(newFile, FileMode.Create, FileAccess.Write));

            pdfInterface.WriteField("stork_eIdentifier", storkPDF.Stork_eIdentifier, pdfStamper);
            pdfInterface.WriteField("stork_surname", storkPDF.Stork_surname, pdfStamper);
            pdfInterface.WriteField("stork_givenName", storkPDF.Stork_givenName, pdfStamper);
            pdfInterface.WriteField("stork_dateOfBirth", storkPDF.Stork_dateOfBirth, pdfStamper);
            pdfInterface.WriteField("stork_textResidence", storkPDF.Stork_textResidence, pdfStamper);
            pdfInterface.WriteField("stork_countryCode", storkPDF.Stork_countryCode, pdfStamper);

            // flatten the form to remove editting options, set it to false
            // to leave the form open to subsequent manual edits
            pdfStamper.FormFlattening = false;

            pdfStamper.Close();
        }

        #endregion

    }
}